NodeId

Struct NodeId 

pub struct NodeId(/* private fields */);
AST Node ID

Implementations

Source

impl NodeId

pub const MAX_INDEX: usize

The maximum representable index value.

pub const CHECKS_MAX_INDEX: bool = true

Whether this index type performs bounds checking.

pub const fn new(value: usize) -> Self

Create a new index from a usize value.

Panics

Panics if value > MAX_INDEX.

pub const fn from_raw(value: NonMaxU32) -> Self

Create an index from a raw NonMaxU32 value.

pub fn from_foreign<F: Idx>(value: F) -> Self

Convert an index from another index type.

pub const unsafe fn from_usize_unchecked(value: usize) -> Self

Create an index from a usize without bounds checking.

SAFETY

The caller must ensure value < (u32::MAX as usize).

pub const unsafe fn from_raw_unchecked(raw: u32) -> Self

Create an index from a raw u32 without bounds checking.

SAFETY

The caller must ensure the value is not u32::MAX.

pub const fn from_usize(value: usize) -> Self

Create an index from a usize with bounds checking.

Panics

Panics if value > MAX_INDEX.

pub const fn index(self) -> usize

Get the index value as a usize.

pub const fn raw(self) -> NonMaxU32

Get the raw NonMaxU32 value.

Source

impl NodeId

pub const DUMMY: Self

Mock node id.

This is used for synthetically-created AST nodes, among other things.

pub const ROOT: Self

Node id of the Program node.