pub struct ReferenceFlags(/* private fields */);Expand description
Describes how a symbol is being referenced in the AST.
There are three general categories of references:
- Values being referenced as values
- Types being referenced as types
- Values being used in type contexts
Values
Whether a reference is considered Read or Write is determined according to ECMA spec.
See comments on Read and Write below.
Counter-intuitively, y in x = y = z is Write only. x = y = z is equivalent to:
var _temp = z;
y = _temp;
x = _temp;See https://github.com/oxc-project/oxc/issues/5165#issuecomment-2488333549 for a runtime test to determine Read/Write operations in a code snippet.
Value as Type
The ValueAsType flag is a temporary marker for references that need to
resolve to value symbols initially, but will ultimately be treated as type references.
This flag is crucial in scenarios like TypeScript’s typeof operator.
For example, in type T = typeof a:
- The reference to ‘a’ is initially flagged with
ValueAsType. - This ensures that during symbol resolution, ‘a’ should be a value symbol.
- However, the final resolved reference’s flags will be treated as a type.
Types
Type references are indicated by Type. These are used primarily in
type definitions and signatures. Types can never be re-assigned, so
there is no read/write distinction for type references.
Implementations
Sourceimpl ReferenceFlags
impl ReferenceFlags
pub const None: Self
Sourcepub const Read: Self
pub const Read: Self
Symbol is being read from as a Value.
Whether a reference is Read is as defined in the spec:
Under Runtime Semantics: Evaluation, when GetValue
is called on a expression, and the expression is an IdentifierReference.
For example:
1. Let lRef be ? Evaluation of Expression.
2. Perform ? GetValue(lRef).Sourcepub const Write: Self
pub const Write: Self
Symbol is being written to as a Value.
Whether a reference is Write is as defined in the spec:
Under Runtime Semantics: Evaluation, when PutValue
is called on a expression, and the expression is an IdentifierReference.
For example:
1. Let lhs be ? Evaluation of LeftHandSideExpression.
2. Perform ? PutValue(lhs, newValue).Sourcepub const ValueAsType: Self
pub const ValueAsType: Self
A value symbol is used in a type context, such as in typeof expressions.
Sourceimpl ReferenceFlags
impl ReferenceFlags
Sourcepub const fn bits(&self) -> u8
pub const fn bits(&self) -> u8
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u8) -> Option<Self>
pub const fn from_bits(bits: u8) -> Option<Self>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u8) -> Self
pub const fn from_bits_truncate(bits: u8) -> Self
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u8) -> Self
pub const fn from_bits_retain(bits: u8) -> Self
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<Self>
pub fn from_name(name: &str) -> Option<Self>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: Self) -> bool
pub const fn intersects(&self, other: Self) -> bool
Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: Self) -> bool
pub const fn contains(&self, other: Self) -> bool
Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn remove(&mut self, other: Self)
pub fn remove(&mut self, other: Self)
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: Self)
pub fn toggle(&mut self, other: Self)
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: Self, value: bool)
pub fn set(&mut self, other: Self, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: Self) -> Self
pub const fn intersection(self, other: Self) -> Self
The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: Self) -> Self
pub const fn union(self, other: Self) -> Self
The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: Self) -> Self
pub const fn difference(self, other: Self) -> Self
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: Self) -> Self
pub const fn symmetric_difference(self, other: Self) -> Self
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> Self
pub const fn complement(self) -> Self
The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl ReferenceFlags
impl ReferenceFlags
Sourcepub const fn iter(&self) -> Iter<ReferenceFlags>
pub const fn iter(&self) -> Iter<ReferenceFlags>
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<ReferenceFlags>
pub const fn iter_names(&self) -> IterNames<ReferenceFlags>
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.
Sourceimpl ReferenceFlags
impl ReferenceFlags
pub const fn read() -> Self
pub const fn write() -> Self
pub const fn read_write() -> Self
Sourcepub const fn is_read_only(&self) -> bool
pub const fn is_read_only(&self) -> bool
The identifier is only read from.
Sourcepub const fn is_write_only(&self) -> bool
pub const fn is_write_only(&self) -> bool
The identifier is only written to. It is not read from in this reference.
Sourcepub fn is_read_write(&self) -> bool
pub fn is_read_write(&self) -> bool
The identifier is both read from and written to, e.g a += 1.
Sourcepub fn is_value_as_type(&self) -> bool
pub fn is_value_as_type(&self) -> bool
Checks if the reference is a value being used in a type context.
pub const fn is_type_only(self) -> bool
pub const fn is_value(&self) -> bool